{ Main state, where most of the application logic takes place.

  Feel free to use this code as a starting point for your own projects.
  This template code is in public domain, unlike most other CGE code which
  is covered by BSD or LGPL (see https://castle-engine.io/license). }
unit GameStateMain;

interface

uses Classes,
  CastleVectors, CastleUIState, CastleComponentSerialize, CastleGlImages,
  CastleUIControls, CastleControls, CastleKeysMouse, CastleRectangles;

type
  { Main state, where most of the application logic takes place. }
  TStateMain = class(TUIState)
  published
    { Components designed using CGE editor.
      These fields will be automatically initialized at Start. }
    LabelFps: TCastleLabel;
  public
    SrcRect, DestRect: TFloatRectangle;
    constructor Create(AOwner: TComponent); override;
    procedure Start; override;
    procedure Update(const SecondsPassed: Single; var HandleInput: Boolean); override;
    function Motion(const Event: TInputMotion): boolean; override;
    procedure Render; override;
  end;

var
  StateMain: TStateMain;
  Image: TDrawableImage;

implementation

uses SysUtils, CastleImages;

{ TStateMain ----------------------------------------------------------------- }

constructor TStateMain.Create(AOwner: TComponent);
begin
  inherited;
  DesignUrl := 'castle-data:/gamestatemain.castle-user-interface';
end;

procedure TStateMain.Start;
begin
  inherited;
  Image := TDrawableImage.Create('castle-data:/pixelpotions-free.png');
  SrcRect := FloatRectangle(0, 0, Image.Width div 2, Image.Height div 2);
  DestRect := FloatRectangle(200, 150, 300, 200);
end;

procedure TStateMain.Update(const SecondsPassed: Single; var HandleInput: Boolean);
begin
  inherited;
  { This virtual method is executed every frame.}
{  Assert(LabelFps <> nil, 'If you remove LabelFps from the design, remember to remove also the assignment "LabelFps.Caption := ..." from code');
  LabelFps.Caption := 'FPS: ' + Container.Fps.ToString;}
end;

function TStateMain.Motion(const Event: TInputMotion): boolean;
var
  X, Y: Single;
  ImgX, ImgY: Integer;
begin
  Result := inherited Motion(Event);
  X := (Event.Position.X - DestRect.Left) / DestRect.Width;
  Y := (Event.Position.Y - DestRect.Bottom) / DestRect.Height;
  if (X >= 0) and (Y >= 0) and (X < 1) and (Y < 1) then
  begin
    ImgX := Round(SrcRect.Width * X - SrcRect.Left);
    ImgY := Round(SrcRect.Width * Y - SrcRect.Left);
    if (ImgX >= 0) and (ImgY >= 0) and (ImgX < Image.Width) and (ImgY < Image.Height) then
    begin
      if (Image.Image as TRGBAlphaImage).PixelPtr(ImgX, ImgY)^.W > 10 then
        LabelFps.Caption := 'OVER PIXEL'
      else
        LabelFps.Caption := 'OUT OF PIXEL'
    end else
      LabelFps.Caption := 'Mouse is within the screen rectangle but outside of the image rectangle, the results are ambiguous. However, even here we can try to play with Image.RepeatS and Image.RepeatT';
  end else
    LabelFps.Caption := 'OUT OF IMAGE';
end;

procedure TStateMain.Render;
begin
  inherited Render;
  Image.Draw(DestRect, SrcRect);
end;

finalization
  FreeAndNil(Image);

end.
